<?php

namespace Moblyze\Asset\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Moblyze\Asset\Http\Requests\AssetRequest;
use Moblyze\Asset\Interfaces\AssetRepositoryInterface;
use Moblyze\Asset\Models\Asset;
use Moblyze\Asset\Forms\Asset as Form;

/**
 * APIController  class for asset.
 */
class AssetAPIController extends BaseController
{

    /**
     * Initialize asset resource controller.
     *
     * @param type AssetRepositoryInterface $asset
     *
     * @return null
     */
    public function __construct(AssetRepositoryInterface $asset)
    {
        parent::__construct();
        $this->repository = $asset;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Asset\Repositories\Criteria\AssetResourceCriteria::class);
    }

    /**
     * Display a list of asset.
     *
     * @return Response
     */
    public function index(AssetRequest $request)
    {
        return $this->repository
            ->setPresenter(\Moblyze\Asset\Repositories\Presenter\AssetPresenter::class)
            ->paginate();
    }

    /**
     * Display asset.
     *
     * @param Request $request
     * @param Model   $asset
     *
     * @return Response
     */
    public function show(AssetRequest $request, Asset $asset)
    {
        return $asset->setPresenter(\Moblyze\Asset\Repositories\Presenter\AssetListPresenter::class);
        ;
    }

    /**
     * Create new asset.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AssetRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('asset::asset.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('asset/asset/' . $asset->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('asset/asset');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the asset.
     *
     * @param Request $request
     * @param Model   $asset
     *
     * @return Response
     */
    public function update(AssetRequest $request, Asset $asset)
    {
        try {
            $data = $request->all();

            $asset->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('asset::asset.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('asset/asset/' . $asset->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('asset/asset/' . $asset->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the asset.
     *
     * @param Model   $asset
     *
     * @return Response
     */
    public function destroy(AssetRequest $request, Asset $asset)
    {
        try {
            $asset->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('asset::asset.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('asset/asset/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('asset/asset/' . $asset->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
